/**
 * @file  DFFileDevice.h
 * @brief Schnittstelle fuer die Klasse CDFFileDevice.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFFILEDEVICE_H_
#define _DFFILEDEVICE_H_

#include "DFFileIntelHex.h"
#include "../Common/ATmega.h"
#include "DFFileRawFirmware.h"
#include "DFFileDisplay.h"
#include "DFFileDeviceText.h"
#include "../Common/aes.h"
#include "../Channel/Channel4ModuleHardware.h"

// Groesse des Verschluesselungsblocks
#define IV_DFFILEDEVICE_SIZE_CRYPT  0x200

// Fester Wert fuer Ablage, um erfolgreiche Entschluesselung sicherzustellen
#define IV_DFFILEDEVICE_CRYPT_FIX   0xDA59EF31
// Ablagewerte fuer Einzelbyte der Cryptblockwerte
#define IV_DFFILEDEVICE_CF_0        43
#define IV_DFFILEDEVICE_CF_1        71
#define IV_DFFILEDEVICE_CF_2        89
#define IV_DFFILEDEVICE_CF_3        36

#define IV_DFFILEDEVICE_SN_0        34
#define IV_DFFILEDEVICE_SN_1        17
#define IV_DFFILEDEVICE_SN_2        98
#define IV_DFFILEDEVICE_SN_3        63

/*
Halten der grundlegenden Dateiinformationen.
Versionsnummer, Quelldateinamen, Blockgroessen
 */
#define IV_DFFILEDEVICE_SIZE_HEADER                   0x200 // Maximale Groesze fuer Headerdaten
// Fuer das PZE-MasterIV
#define IV_DFFILEDEVICE_VERSION_PZE                   0x00010000
// Fuer das ZK-MasterIV
#define IV_DFFILEDEVICE_VERSION_ZK                    0x00020000
// Fuer das BDE-MasterIV
#define IV_DFFILEDEVICE_VERSION_BDE                   0x00030000
// Fuer das Mobile-MasterIV
#define IV_DFFILEDEVICE_VERSION_MOBIL                 0x00040000
// Fuer das MDE-BoxIV
#define IV_DFFILEDEVICE_VERSION_MDE                   0x00050000
// Fuer das AE-MasterIV
#define IV_DFFILEDEVICE_VERSION_AE                    0x00060000
// Fuer das TimeboyIV
#define IV_DFFILEDEVICE_VERSION_TIMEBOY               0x00070000
// Fuer mehrere Geraetetypen (Information liegt in der Info-Struktur)
#define IV_DFFILEDEVICE_VERSION_MULTIPLE              0x00080000
// Fuer das Exklusiv-Line IV
#define IV_DFFILEDEVICE_VERSION_EXKLUSIV              0x00100000
// Fuer das Flex-MasterIV
#define IV_DFFILEDEVICE_VERSION_FLEX                  0x00200000
// Fuer das Timeboy-Mobil-PZE
#define IV_DFFILEDEVICE_VERSION_TIMEBOY_MOBIL_PZE     0x00400000


#pragma pack(1)

typedef struct SDFSoftwareInfo {
	unsigned char size;			    // Strukturgroesse (gleich Version der Info)
	unsigned short device;			// Geraetetyp
									#define DEVICE_PZE                               (0x0001)
									#define DEVICE_ZK                                (0x0002)
									#define DEVICE_BDE                               (0x0004)
									#define DEVICE_MOBIL                             (0x0008)
									#define DEVICE_MDEBOX                            (0x0010)
									#define DEVICE_AE                                (0x0020)
									#define DEVICE_TIMEBOY                           (0x0040)
									#define DEVICE_EXKLUSIV                          (0x0080)
									#define DEVICE_FLEX                              (0x0100)
									#define DEVICE_TIMEBOY_MOBIL_PZE                 (0x0200)
									#define DEVICE_USART0_TO_USART1                  (0x1000)
									#define DEVICE_FURUNO_GPS_MONITOR_BYPASS         (0x2000)
	unsigned short board;			// Hardware
									#define BOARD_10                                 (0x0001)
									#define BOARD_11                                 (0x0002)
									#define BOARD_12                                 (0x0004)
									#define BOARD_13                                 (0x0008)
									#define BOARD_14                                 (0x0010)
									#define BOARD_20                                 (0x0020)
									#define BOARD_21                                 (0x0040)
									#define BOARD_30                                 (0x0080)
	unsigned char display;			// Anzeige
									#define DISPLAY_NONE                             (0x00)
									#define DISPLAY_LEHNER_SWAP                      (0x01)
									#define DISPLAY_LEHNER                           (0x02)
									#define DISPLAY_NEUMUELLER                       (0x04)
	unsigned char keypad;			// Tastatur
									#define KEYPAD_NONE                              (0x00)
									#define KEYPAD_PZE                               (0x01)
									#define KEYPAD_BDE                               (0x02)
									#define KEYPAD_AE                                (0x04)
									#define KEYPAD_PZE_BASIC                         (0x08)
									#define KEYPAD_TIMEBOY                           (0x10)
									#define KEYPAD_FLEX                              (0x20)
									#define KEYPAD_TIMEBOY_MOBIL_PZE                 (0x40)
	unsigned char radio;            // Funk
									#define RADIO_NONE                               (0x00)
									#define RADIO_BLUETOOTH                          (0x01)
									#define RADIO_ZIGBEE                             (0x02)
	unsigned char mmc;              // MM Card / USB Stick
									#define MMC_NONE                                 (0x00)
									#define MMC_MEMORY                               (0x01)
									#define MMC_DISK                                 (0x02)
									#define MMC_USBHOST                              (0x04)
	unsigned char modem;            // Mobilfunk
									#define MODEM_NONE                               (0x00)
									#define MODEM_MC39                               (0x01)
									#define MODEM_MC55                               (0x02)
	unsigned char rs485;            // RS485 Kommunikation
									#define RS485_NONE                               (0x00)
									#define RS485_TIP                                (0x01)
	unsigned char tcpip;            // TCP/IP | WLAN
									#define TCPIP_NONE                               (0x00)
									#define TCPIP_XPORT                              (0x01)
									#define TCPIP_MATCHPORT                          (0x02)
	unsigned char usb;              // USB Kommunikation
									#define USB_NONE                                 (0x00)
									#define USB_TIP                                  (0x01)
	unsigned char access;           // Zutrittskontrolle mit externen Lesertyp
									#define ACCESS_NONE                              (0x00)
									#define ACCESS_GIS                               (0x01)
									#define ACCESS_PHG                               (0x02)
									#define ACCESS_PROTECT                           (0x04)
	unsigned char dcf77;            // Funkuhr oder Barcode
									#define DCF77_NONE                               (0x00)
									#define DCF77_HKW                                (0x01)
									#define DCF77_BARCODE                            (0x02)
                                    #define DCF77_BARCODE_ZEBEX                      (0x04)
	unsigned char docking;          // Anbindung TimeboyIII
									#define DOCKING_NONE                             (0x00)
									#define DOCKING_ENABLE                           (0x01)
	unsigned char fingerprint;      // Fingerprintmodul 
									#define FP_NONE                                  (0x00)
									#define FP_BIOKEY                                (0x01)
	unsigned char gps;              // GPS Modul
									#define GPS_NONE                                 (0x00)
									#define GPS_FURUNO                               (0x01)
									#define GPS_LEA                                  (0x02)
									#define GPS_MOUSE_LEA                            (0x04)
	unsigned char iomodule;         // Anbindung IO-Modul
									#define IOMODULE_NONE                            (0x00)
									#define IOMODULE_ENABLE                          (0x01)
	unsigned short transponder;     // Transponderleser
									#define TP_NONE                                  (0x0000)
									#define TP_TSR30                                 (0x0001)
									#define TP_TSR32                                 (0x0002)
									#define TP_PRIMO100                              (0x0004)
									#define TP_MIFARE                                (0x0008)
									#define TP_IBUTTON_EXT                           (0x0010)
									#define TP_SMARTRELAIS                           (0x0020)
									#define TP_NEDAP                                 (0x0040)
									#define TP_IBUTTON_INT                           (0x0080)
									#define TP_HID_PROXKEY                           (0x0100)
									#define TP_MIFARE_DESFIRE                        (0x0200)
									#define TP_LEGIC_ADVANT                          (0x0400)
                  #define TP_TSR21                                 (0x0800)
									#define TP_HID_ICLASS                            (0x1000)
								  #define TP_TB_MIFARE                             (0x2000)
  								#define TP_TB_LEGIC                              (0x4000)
	unsigned char analog_in;        // Analoge Eingaenge
									#define ANALOG_IN_NONE                           (0x00)
									#define ANALOG_IN_2                              (0x03)
	unsigned char digital_in;       // Digitale Eingaenge
									#define DIGITAL_IN_NONE                          (0x00)
									#define DIGITAL_IN_2                             (0x03)
									#define DIGITAL_IN_6                             (0x3F)
									#define DIGITAL_IN_8                             (0xFF)
	unsigned char digital_out;      // Digitale Ausgaenge (Relais)
									#define DIGITAL_OUT_NONE                         (0x00)
									#define DIGITAL_OUT_2                            (0x03)
									#define DIGITAL_OUT_4                            (0x0F)
	unsigned char trace;            // 
									#define TRACE_NONE                               (0x00)
//###########################
// Ab Version 2
	unsigned char comserver;        // Com-Server Modus
                                    #define COMSERVER_NONE                           (0x00)
                                    #define COMSERVER_ENABLE                         (0x01)

	unsigned char transponder_write;// Beschreiben von Transpondern im Transpondermenue
                                    #define TP_WRITE_NONE                            (0x00)
                                    #define TP_WRITE_ENABLE                          (0x01)

	unsigned char http;             // Versenden von Datensaetzen per HTTP
                                    #define HTTP_NONE                                (0x00)
                                    #define HTTP_ENABLE                              (0x01)

	unsigned char language;         // Firmware benoetigt Textdaten zum Ordnungsgemaessen Betrieb
                                    #define LANGUAGE_NONE                            (0x00)
                                    #define LANGUAGE_ENABLE                          (0x01)

	unsigned char biokey_update;    // Update des Integrierten Biokeymoduls ueber die Firmware moeglich
									#define BIOKEY_UPDATE_NONE                       (0x00)
									#define BIOKEY_UPDATE_ENABLE                     (0x01)
//###########################
// Ab Version 3
	unsigned short setup_options;   // Unterstuetzte Bestandteile des Setups
                                    #define SETUP_OPTION_MASK                        (0x0001) // Maske der unterstuetzten Setupoptionen
                                    #define SETUP_OPTION_ALL                         (0xFFFF)
                                    #define SETUP_OPTION_IOCONTROL                   (0x0001) // Signalverarbeitung

//###########################
// Ab Version 4
	unsigned char touch;            // Touchpanel
									#define TOUCH_NONE                               (0x00)
									#define TOUCH_SMALL                              (0x01)
									#define TOUCH_BIG                                (0x02)

	unsigned char sensor;           // Sensoren
									#define SENSOR_NONE                              (0x00)
									#define SENSOR_VIBRATION                         (0x01)
									#define SENSOR_SPEEDUP                           (0x02)

	unsigned char battery;          // Lithium Batterie
									#define BATTERY_NO                               (0x00)
									#define BATTERY_YES                              (0x01)

	unsigned char charge;           // Ladeschaltung
									#define CHARGE_NO                                (0x00)
									#define CHARGE_YES                               (0x01)

} TDFSoftwareInfo;
#define sizeof_SDFSoftwareInfo_1    25 // Strukturgroesse der Version 1 ist: 25Byte
#define sizeof_SDFSoftwareInfo_2    30 // Strukturgroesse der Version 2 ist: 30Byte
#define sizeof_SDFSoftwareInfo_3    32 // Strukturgroesse der Version 3 ist: 32Byte
#define sizeof_SDFSoftwareInfo_4    36 // Strukturgroesse der Version 4 ist: 36Byte

struct SDFFileDeviceHeader
{
	unsigned int versionFile; // GeraetetypDateiversion (Major 1 -> PZE Master IV, Major 2 -> Zutritt Master IV, Major 3 -> BDE Master IV)
	unsigned short sizeData;   // Groesse der Headstruktur, gleichzeitig Version: sizeof(SDFFileDeviceHeader), wird auch benoetigt fuer die Berechnung der Position der crcHead - Variablen
	char timestamp[20];        // Datetime vom Build, im Format "2005-01-25 07:00:00\0"
	unsigned int crcProgram;  // 32bit CRC fuer Programm-Flash
	char versionFirmware[32];  // Firmwareversion als String, im Beispielformat: "04.00.23.03\0"
	SDFSoftwareInfo softwareInfo; // Softwareinformation zur enthaltenen Anwendungsfirmware.
	//...
	unsigned short crcHead;    // 16bit CRC fuer Head (Immer letzter Wert in der Struktur; 
                               // Berechnungsmenge: (sizeData - sizeof(unsigned short)), also ohne sich selbst!)
};

struct SDFFileDeviceHeader1
{
	unsigned int versionFile; // Dateiversion (Major 1 -> PZE Master IV, Major 2 -> Zutritt Master IV, Major 3 -> BDE Master IV)
	unsigned short sizeData;   // Groesse der Headstruktur: sizeof(SDFFileDeviceHeader), wird benoetigt fuer die Berechnung der Position der crcHead - Variable
	char timestamp[20];        // Datetime vom Build, im Format "2005-01-25 07:00:00\0"
	unsigned int crcProgram;  // 32bit CRC fuer Programm-Flash
	char versionFirmware[32];  // Firmwareversion als String, im Beispielformat: "04.00.23.03\0"
	//...
	unsigned short crcHead;    // 16bit CRC fuer Head (Immer letzter Wert in der Struktur; 
	// Berechnungsmenge: (sizeData - sizeof(unsigned short)), also ohne sich selbst!)
};
#pragma pack()

class CDFFileDevice : public CDFFileIntelHex
{
public:
	CDFFileDevice();
	virtual ~CDFFileDevice();

	void Reset();
	
	inline unsigned int GetHeaderVersionFile() { return m_header->versionFile; };
	inline unsigned int GetHeaderVersionTimestamp(char *timestamp) { return sprintf(timestamp, "%s", m_header->timestamp); };
	inline unsigned int GetHeaderVersionFirmware(char *versionFirmware) { return sprintf(versionFirmware, "%s", m_header->versionFirmware); };
	inline SDFSoftwareInfo GetHeaderSoftwareInfo() { return m_header->softwareInfo; }
	inline unsigned char IsCrypt() { return m_isCrypt; };
	inline void SetSerial(unsigned int numSerial) { m_numSerial = numSerial; };
	inline unsigned int GetSerial() { return m_numSerial; };
	inline unsigned int GetSerialCrypt() 
	{ 
		unsigned int numSerial;
		numSerial = m_crypt[IV_DFFILEDEVICE_SN_0];
		numSerial |= (unsigned int)m_crypt[IV_DFFILEDEVICE_SN_1] << 8;
		numSerial |= (unsigned int)m_crypt[IV_DFFILEDEVICE_SN_2] << 16;
		numSerial |= (unsigned int)m_crypt[IV_DFFILEDEVICE_SN_3] << 24;
		return numSerial;
	}

	SDFFileDeviceHeader GetHeader(char *fileName);
	int CheckVersionFile(SDFFileDeviceHeader &header);
	unsigned int GetVersionFile(char *fileName);
	unsigned int GetVersionFirmware(char *fileName, char *versionFirmware); // max. 32 Zeichen Firmwareversion
	int Load(char *fileName);
	int Save(char *fileName, SDFFileDeviceHeader &header);
	int Build(char *fileDevice, SDFFileDeviceHeader &header, char *fileFirmware, char *fileDisplayData, char *fileDeviceText);

	SDFSoftwareInfo SWInfoStringToStruct(char *data)
	{
		SDFSoftwareInfo swInfo;
		memset(&swInfo, 0, sizeof(SDFSoftwareInfo));

		// # durch : ersetzen
		data[INTELHEX_START] = INTELHEX_START_SIGN;    // start of record
		// CRC pruefen
		if (!CheckCrc(data)) {

			return swInfo;
		}
		
		unsigned char i, count = GetCount(data);
		// spaetere Groessenanderung vorbehalten!
		for (i=0; (i < count) && i < sizeof(SDFSoftwareInfo); i++) {
			
			((unsigned char*)&swInfo)[i] = GetData8(data, i);
		}

		return CheckSoftwareInfo(swInfo);
	}

	char *SWInfoStructToString(char *data, SDFSoftwareInfo &swInfo)
	{
		if (MakeDataLine(0, (unsigned char*)&swInfo, sizeof(SDFSoftwareInfo), data) == 0) {

			return NULL;
		}
		// : durch # ersetzen
		data[INTELHEX_START] = '#';    // start of record

		return data;
	}

	SDFSoftwareInfo CheckSoftwareInfo(SDFSoftwareInfo swInfo)
	{
		if (swInfo.size > sizeof(SDFSoftwareInfo)) {

			// Zu aktuell fuer diese Softwareversion
			memset(&swInfo, 0, sizeof(SDFSoftwareInfo));
			return swInfo;
		}
		
		// Anpassungen fuer neuere Versionen durchfuehren
		if (swInfo.size < sizeof_SDFSoftwareInfo_2) {

			// Auf Version 2 anpassen
			swInfo.comserver         = COMSERVER_ENABLE; // Comserver aktiv
			swInfo.transponder_write = (swInfo.transponder > 0) ? TP_WRITE_ENABLE : TP_WRITE_NONE; // Transponder schreiben (koennen eigentlich nicht alle)
			swInfo.http              = (swInfo.modem > 0) ? HTTP_ENABLE : HTTP_NONE; // HTTP-Protokoll, wenn Modem vorhanden.
			swInfo.language          = LANGUAGE_ENABLE; // Sprachdatei wird untersttzt. Bedeutet nicht, dass in alten Geraetedateien diese da sein mssen.
			swInfo.biokey_update     = (swInfo.fingerprint > 0) ? BIOKEY_UPDATE_ENABLE : BIOKEY_UPDATE_NONE; // Biokey updaten, wenn Fingerprint vorhanden.
		} 

		// Anpassungen fuer neuere Versionen durchfuehren
		if (swInfo.size < sizeof_SDFSoftwareInfo_3) {

			// Auf Version 3 anpassen
			swInfo.setup_options      = SETUP_OPTION_ALL; // Alle Optionen des Setups werden untersttzt
		} 

		// Anpassungen fuer neuere Versionen durchfuehren
		if (swInfo.size < sizeof_SDFSoftwareInfo_4) {

			// Auf Version 4 anpassen
			swInfo.touch   = TOUCH_NONE;
			swInfo.sensor  = SENSOR_NONE;
			swInfo.battery = BATTERY_NO;
			swInfo.charge  = CHARGE_NO;
		} 

		return swInfo;
	}

	int CompareInfos(TDFHardwareInfo hwInfo, TDFSoftwareInfo swInfo);
	int GetInfos(TDFSoftwareInfo swInfo, char *info);

	unsigned int GetSizeData();
	inline CDFFileDisplay *GetFileDisplay() { return &m_fileDisplay; };
	inline CDFFileDeviceText *GetFileDeviceText() { return &m_fileDeviceText; };
	inline CATmega *GetController() { return m_fileRawFirmware.GetController(); };

	// Verschluesseln
	int Encrypt(unsigned int numSerial);
	// Entschluesseln
	int Decrypt();

	CDFFileRawFirmware m_fileRawFirmware;
	char m_fileName[260];
protected:
	// Der Dateiheader wird ueber einen Puffer verwaltet, somit ist eine
	// vergroesserte Struktur auch von aelteren Softwarestaenden lesbar und
	// eine Pruefung des crcHead durch info:sizeData moeglich.
	unsigned char m_headBuf[IV_DFFILEDEVICE_SIZE_HEADER];
	SDFFileDeviceHeader *m_header;
	CDFFileDisplay m_fileDisplay;
public:
	CDFFileDeviceText m_fileDeviceText;

protected:
	unsigned char m_crypt[IV_DFFILEDEVICE_SIZE_CRYPT];
	unsigned char m_isCrypt;
	unsigned int m_numSerial;
};

#endif

